@@ -1,4 +1,4 @@
-/* $Id: appointment.h 1466 2003-12-16 11:40:48Z twogood $ */
+/* $Id: appointment.h 3713 2009-03-17 14:32:11Z mark_ellis $ */
 #ifndef __appointment_h__
 #define __appointment_h__
 
@@ -23,7 +23,8 @@ bool rra_appointment_to_vevent(
     size_t data_size,
     char** vevent,
     uint32_t flags,
-    struct _RRA_Timezone* tzi);
+    struct _RRA_Timezone* tzi,
+    const char *codepage);
 
 bool rra_appointment_from_vevent(
     const char* vevent,
@@ -31,7 +32,8 @@ bool rra_appointment_from_vevent(
     uint8_t** data,
     size_t* data_size,
     uint32_t flags,
-    struct _RRA_Timezone* tzi);
+    struct _RRA_Timezone* tzi,
+    const char *codepage);
 
 #define rra_appointment_free_vevent(p)  if (p) free(p)
 #define rra_appointment_free_data(p)    if (p) free(p)
