@@ -1,4 +1,4 @@
-/* $Id: contact.h 2129 2005-07-22 07:30:57Z twogood $ */
+/* $Id: contact.h 3713 2009-03-17 14:32:11Z mark_ellis $ */
 #ifndef __contact_h__
 #define __contact_h__
 
@@ -31,14 +31,16 @@ bool rra_contact_to_vcard(
 		const uint8_t* data, 
 		size_t data_size,
 		char** vcard,
-		uint32_t flags);
+		uint32_t flags,
+		const char *codepage);
 
 bool rra_contact_from_vcard(
 		const char* vcard, 
 		uint32_t* id,
 		uint8_t** data, 
 		size_t* data_size,
-		uint32_t flags);
+		uint32_t flags,
+		const char *codepage);
 #endif /* SWIG */
 
 struct FieldStrings
