@@ -1,4 +1,4 @@
-/* $Id: generator.c 2065 2005-05-13 11:32:26Z twogood $ */
+/* $Id: generator.c 3713 2009-03-17 14:32:11Z mark_ellis $ */
 #define _GNU_SOURCE 1
 #include "generator.h"
 #include "dbstream.h"
@@ -158,7 +158,7 @@ bool generator_run(Generator* self)
     }
     else
     {
-      /*synce_trace("Unhandled property id: %04x", id);*/
+      synce_trace("Unhandled property id: %04x", id);
     }
   }
 
