@@ -1,4 +1,4 @@
-/* $Id: rrac.c 2752 2007-03-14 22:14:01Z jagow $ */
+/* $Id: rrac.c 3749 2009-04-12 10:23:37Z mark_ellis $ */
 #include "rrac.h"
 #include "rrac_packet.h"
 #include <synce_log.h>
@@ -7,7 +7,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/param.h>
-#include <rapi.h>
 
 #define LETOH16(x)  x = letoh16(x)
 #define LETOH32(x)  x = letoh32(x)
@@ -145,29 +144,35 @@ bool rrac_set_command_69_callback(/*{{{*/
 
 /* create server socket, call CeStartReplication(), accept command and
  * data channel */
-bool rrac_connect(RRAC* rrac)/*{{{*/
+bool rrac_connect(RRAC* rrac, RapiConnection *connection)/*{{{*/
 {
   HRESULT hr;
-	rrac->server = synce_socket_new();
+  const char *ip_addr = NULL;
 
-	if (!synce_socket_listen(rrac->server, NULL, RRAC_PORT))
-		goto fail;
+  rrac->server = synce_socket_new();
 
-	hr = CeStartReplication();
+  ip_addr = rapi_connection_get_local_ip(connection);
+
+  if (!synce_socket_listen(rrac->server, ip_addr, RRAC_PORT))
+    goto fail;
+
+  if (connection)
+    rapi_connection_select(connection);
+  hr = CeStartReplication();
   if (FAILED(hr))
   {
     synce_error("CeStartReplication failed: %s", synce_strerror(hr));
     goto fail;
   }
 
-	rrac->cmd_socket   = synce_socket_accept(rrac->server, NULL);
-	rrac->data_socket  = synce_socket_accept(rrac->server, NULL);
+  rrac->cmd_socket   = synce_socket_accept(rrac->server, NULL);
+  rrac->data_socket  = synce_socket_accept(rrac->server, NULL);
 
-	return true;
+  return true;
 
 fail:
-	rrac_disconnect(rrac);
-	return false;
+  rrac_disconnect(rrac);
+  return false;
 }/*}}}*/
 
 /* disconnect sockets */
