@@ -1,8 +1,9 @@
-/* $Id: syncmgr.h 2121 2005-07-08 19:25:22Z twogood $ */
+/* $Id: syncmgr.h 3750 2009-04-14 12:43:36Z mark_ellis $ */
 #ifndef __rra_syncmgr_h__
 #define __rra_syncmgr_h__
 
 #include <synce.h>
+#include <rapi.h>
 
 /* Constants for use with rra_syncmgr_type_from_name() */
 #define RRA_SYNCMGR_TYPE_APPOINTMENT  "Appointment"
@@ -51,7 +52,7 @@ void rra_syncmgr_destroy(RRA_SyncMgr* self);
    Connect or disconnect from device 
  */
 
-bool rra_syncmgr_connect(RRA_SyncMgr* self);
+bool rra_syncmgr_connect(RRA_SyncMgr* self, RapiConnection *connection);
 void rra_syncmgr_disconnect(RRA_SyncMgr* self);
 bool rra_syncmgr_is_connected(RRA_SyncMgr* self);
 
@@ -60,9 +61,9 @@ bool rra_syncmgr_is_connected(RRA_SyncMgr* self);
  */
 
 uint32_t rra_syncmgr_get_type_count(RRA_SyncMgr* self);
-RRA_SyncMgrType* rra_syncmgr_get_types(RRA_SyncMgr* self);
-RRA_SyncMgrType* rra_syncmgr_type_from_id(RRA_SyncMgr* self, uint32_t type_id);
-RRA_SyncMgrType* rra_syncmgr_type_from_name(RRA_SyncMgr* self, const char* name);
+const RRA_SyncMgrType* rra_syncmgr_get_types(RRA_SyncMgr* self);
+const RRA_SyncMgrType* rra_syncmgr_type_from_id(RRA_SyncMgr* self, uint32_t type_id);
+const RRA_SyncMgrType* rra_syncmgr_type_from_name(RRA_SyncMgr* self, const char* name);
 
 /*#define RRA_SYNCMGR_INVALID_TYPE_ID   ((uint32_t)0xffffffff)*/
 
