@@ -1,4 +1,4 @@
-/* $Id: task.h 1466 2003-12-16 11:40:48Z twogood $ */
+/* $Id: task.h 3713 2009-03-17 14:32:11Z mark_ellis $ */
 #ifndef __task_h__
 #define __task_h__
 
@@ -23,7 +23,8 @@ bool rra_task_to_vtodo(
     size_t data_size,
     char** vtodo,
     uint32_t flags,
-    struct _RRA_Timezone* tzi);
+    struct _RRA_Timezone* tzi,
+    const char *codepage);
 
 bool rra_task_from_vtodo(
     const char* vtodo,
@@ -31,7 +32,8 @@ bool rra_task_from_vtodo(
     uint8_t** data,
     size_t* data_size,
     uint32_t flags,
-    struct _RRA_Timezone* tzi);
+    struct _RRA_Timezone* tzi,
+    const char *codepage);
 
 #define rra_task_free_vtodo(p)  if (p) free(p)
 #define rra_task_free_data(p)   if (p) free(p)
